function [u, x] = GreedyNonlinear(F, Ahat, xbar, beta, lambda, T, x0, w)
% This function is used to implement Greedy method when the environment evolves by nonlinear dynamic. 
% This code use the "lsqnonlin" function in Optimization Toolbox.
% In this code, we shift vector's time index by 1. For example, 
% xbar(t+1,t'+1) corresponds to xbar_{t'|t} in paper.
%
% inputs:
% F(x,u,w,mode) is a function handle with three modes for environment dynamic \bfx_{t+1} = F(\bfx_t, u_t, w_t, 1)
% when mode = 1, F(x,u,w,1) returns F(x,u,w)
% when mode = 2, F(x,u,w,2) returns DxF(x,u,w), DxF_{ij} corresponds to dF_i/dxj
% when mode = 3, F(x,u,w,3) returns DuF(x,u,w), DuF_{i1} corresponds to dF_i/du 
% 
% Ahat: (p+1)-by-(p+1) forecast matrix, the forecaster's AR(p) model
%
% xbar: (T+1)-by-(T+1) matrix, scalar version of adversarial target.
% xbar(t+1,t'+1) corresponds to xbar_{t'|t} in paper. For xbar(t+1,t'+1) 
% that is: t < 0 or t>=t' or t=T, xbar(t+1,t'+1) should be set to 0.
%
% beta: (T+1)-by-(T+1) matrix, nonzero entry of Q matrix, beta(t+1,t'+1)
% corresponds to the nonzero entry in the quadratic cost matrix Q_{t'|t} in 
% paper. For beta(t+1,t'+1) that is: t<0 or t>=t' or t=T, beta(t+1,t'+1)
% should be set to 0.
%
% lambda>0: adversarial effort parameter
%
% T: prediction time horizon
%
% x0: the scalar initial value
%
% w: 1-by-T the noise sequence w_t, t = 0,1,...,T-1
%
%
%
% outputs:
% u: 1-by-T array for attack action sequence u_t, t = 0, 1, ... , T-1
% x: (p+1)-by-(T+1) array for environment state sequence \bfx_t, t = 0,1,...,T



% retrieve p
p = length(Ahat(1,:))-1;

% u is a 1-by-T array for u_t, t = 0,1,...,T-1
% the u_{T-1} is already correct			
u = zeros(1, T);

% A (p+1)-by-(T+1) array for \bfx_t, t = 0,1,...,T
x = zeros(p+1, T+1);
x(1, :) = ones(1, T+1);
x(2, 1) = x0;

% specify Jacobian in lsqnonlin
options = optimoptions('lsqnonlin','SpecifyObjectiveGradient',true);

for t = 0 : T-2

	% call lsqnonlin function in Matlab
	% initialize u at 0
	u(t + 1) = lsqnonlin(@(u) fun(u, x(:, t+1), t, F, Ahat, xbar, beta, lambda, T, p), 0);

	% implement the first action
	x(:,t + 2) = F(x(:, t + 1), u(t + 1), w(t + 1),1);
end

end


function [FF, D] = fun(u, xt, t, F, Ahat, xbar, beta, lambda, T, p)
% this function is used to construct the function that lsqnonlin can use
% initialize F and D
FF = zeros(T-t, 1);
D = zeros(T-t, 1);

% set beta vector
betabf = zeros(p+1, T+1);
betabf(2, t+3:T+1) = beta(t+2, t+3:T+1).^0.5;

% set \bar \bfx
bfxbar = zeros(p+1,T+1);
bfxbar(2, t+3:T+1) = xbar(t+2, t+3:T+1);

for tp = t+2 : T
	FF(tp - t - 1) = betabf(:, tp+1)' * (Ahat^(tp - (t+1)) * F(xt, u, 0, 1) - bfxbar(:, tp+1));
	D(tp - t - 1) = betabf(:, tp+1)' * (Ahat^(tp - (t+1)) * F(xt, u, 0, 3) - bfxbar(:, tp+1));
end

FF(T-t) = lambda^0.5 * u;
D(T-t) = lambda^0.5;

end
