function PHI = LQRStateAttack(A,Ahat,xbar,beta,lambda,T)
% function phi = LQRStateAttack(A,Ahat,xbar,beta,lambda,T)
% This function computes the adversary's state attack when the environment 
% dynamics is linear (Theorem 1). 
% It returns a function handle of policy function phi. 
% In this code, we shift vector's time index by 1. For example, 
% xbar(t+1,t'+1) corresponds to xbar_{t'|t} in paper. However, matrix 
% dimension remain the same for all of the matrices in Theorem 1. 
%
%
% Input:
% A: (p+1)-by-(p+1) matrix, in the environment dynamics:
% x_{t+1} = F(x_t, u_t, w_t) := A x_t + B (u_t + w_t)
%
% Ahat: (p+1)-by-(p+1) matrix, the forecaster's AR(p) model
%
% xbar: (T+1)-by-(T+1) matrix, scalar version of adversarial target.
% xbar(t+1,t'+1) corresponds to xbar_{t'|t} in paper. For xbar(t+1,t'+1) 
% that is: t < 0 or t>=t' or t=T, xbar(t+1,t'+1) should be set to 0.
%
% beta: (T+1)-by-(T+1) matrix, nonzero entry of Q matrix, beta(t+1,t'+1)
% corresponds to the nonzero entry in the quadratic cost matrix Q_{t'|t} in 
% paper. For beta(t+1,t'+1) that is: t<0 or t>=t' or t=T, beta(t+1,t'+1)
% should be set to 0.
%
% lambda>0: adversarial effort parameter
%
% T: prediction time horizon
%
%
% Output:
% phi: function handle for ploicy function. One can call it as:
%
% ut = phi(t,xt)
%
% xt is the (p+1)-by-1 state vector, t is time index



% retrieve p
p = length(A(:,1)) - 1;

% define B
B = zeros(p+1,1);
B(2) = 1;

% initialize P, q sequence:
% P(:,:,t+1) corresponds to P_t
% q(:,t+1) corrosponds to q_t
P = zeros(p+1, p+1, T+1);
q = zeros(p+1,T+1);

% P_T, q_T are already correct now

% compute P, q sequence for t = T-1, ..., 1
for t = T-1:-1:1
	% set Q_{t+1:T | t}
    % Q(:,:,t'+1) corresponds to Q_{t'|t}
	Q = zeros(p+1,p+1,T+1);
	Q(2,2,t+2:T+1) = beta(t+1, t+2:T+1);
	% compute the first part of P_t
	sum1 = zeros(p+1,p+1);
	for tau = t+1 : T
		sum1 = sum1 + (Ahat^(tau-t))'*Q(:,:,tau+1)*(Ahat^(tau-t));
	end
	% compute P_t
	P(:,:,t+1) = sum1 + A'*(eye(p+1) + P(:,:,t+2)*B*B'/lambda)^(-1)*P(:,:,t+2)*A;
	% set bfxbar, only the second entry will be used
    % bfbar(:,t'+1) corresponds to barbfx_{t'|t}
	bfxbar = zeros(p+1,T+1);
	bfxbar(2,t+2:T+1) = xbar(t+1, t+2:T+1);
	% compute the first part of q_t
	sum2 = zeros(p+1,1);
	for tau = t+1 : T
		sum2 = sum2 + (Ahat^(tau-t))'*Q(:,:,tau+1)*bfxbar(:,tau+1);
	end
	% compute q_t
	q(:,t+1) = -2*sum2 + A'*q(:,t+2) - A'*P(:,:,t+2)'*B*B'*q(:,t+2)/(lambda + B'*P(:,:,t+2)*B);
end
% initialize phi, start at 0
% phi{t+1} corresponds to phi_t
phi = cell(T,1);
% compute phi_t
for t = 1:T
	phi{t} = @(z)-(B'*q(:,t+1) + 2*B'*P(:,:,t+1)*A*z)/(2*(lambda + B'*P(:,:,t+1)*B));
end
% restructure phi
PHI = @(t,xt) phi{t+1}(xt);
end





