function [u, x] = MPC(F, l, Ahat, xbar, beta, lambda, T, x0, w, maxiter, tol)
% This function is used to implement MPC framework 
% In this code, we shift vector's time index by 1. For example, 
% xbar(t+1,t'+1) corresponds to xbar_{t'|t} in paper.

% inputs:
% F(x,u,w,mode) is a function handle with three modes for dynamic \bfx_{t+1} = F(\bfx_t, u_t, w_t, 1)
% when mode = 1, F(x,u,w,1) returns F(x,u,w)
% when mode = 2, F(x,u,w,2) returns DxF(x,u,w), DxF_{ij} corresponds to dF_i/dxj
% when mode = 3, F(x,u,w,3) returns DuF(x,u,w), DuF_{i1} corresponds to dF_i/du 
% 
% l: looks l steps ahead. For example, at time tau, u_tau,...,u_{tau+l-1} is considered. When time index goes beyond T, cut the tail
%
% Ahat: (p+1)-by-(p+1) matrix, the forecaster's AR(p) model
%
% xbar: (T+1)-by-(T+1) matrix, scalar version of adversarial target.
% xbar(t+1,t'+1) corresponds to xbar_{t'|t} in paper. For xbar(t+1,t'+1) 
% that is: t < 0 or t>=t' or t=T, xbar(t+1,t'+1) should be set to 0.
%
% beta: (T+1)-by-(T+1) matrix, nonzero entry of Q matrix, beta(t+1,t'+1)
% corresponds to the nonzero entry in the quadratic cost matrix Q_{t'|t} in 
% paper. For beta(t+1,t'+1) that is: t<0 or t>=t' or t=T, beta(t+1,t'+1)
% should be set to 0.
%
% lambda>0: adversarial effort parameter
%
% T: prediction time horizon
%
% x0: the scalar initial value
%
% w: 1-by-T the noise sequence w_t, t = 0,1,...,T-1
%
% maxiter: maximum iteration for ILQR
%
% tol: if \| \delta u\|^2/length(\delta u), break
%
%
% outputs:
% u: 1-by-T array for action sequence u_t, t = 0, 1, ... , T-1
% x: (p+1)-by-(T+1) array for state sequence \bfx_t, t = 0,1,...,T



% retrieve p
p = length(Ahat(1,:))-1;

% A 1-by-T array for u_t, t = 0,1,...,T-1
% the u_{T-1} is already correct			
u = zeros(1, T);

% A (p+1)-by-(T+1) array for \bfx_t, t = 0,1,...,T
x = zeros(p+1, T+1);
x(1, :) = ones(1, T+1);
x(2, 1) = x0;

% define L()
L = @(tau) min(tau + l - 1, T - 2);

ustar = zeros(1,L(0) + 1);

% run MPC
for tau = 0 : T-2
	% call ILQR to solve the first step of MPC
	% use the solution from last MPC step to initialize the first length(ustar)-1 element for u 
	uinit = zeros(1,L(tau) + 1 - tau);
	uinit(1:length(ustar)-1) = ustar(2:end);
	ustar = ILQR(F, uinit, x(:,tau+1), tau, L(tau), T, Ahat, xbar, beta, lambda, maxiter, tol);
	
	% keep the first action
	u(tau + 1) = ustar(1);

	% implement the first action
	x(:,tau + 2) = F(x(:, tau + 1), u(tau + 1), w(tau + 1),1);
end

end

