% this code is used to generate plot 'figexper4_MPCtoy.eps' for the toy example in section 4.4.
% this code shows results for paired t-test between realized cost under MPC and Greedy policy

clear;

% used fixed random seed to generate noise sequence
rng(1);

% environment dynamic, initial state, model order, time horizon
F = @(x, u, w, mode) Ftoy(x, u, w, mode);
x0 = 3;
p = 1;
T = 10;

% simulate state sequence and estimate forecaster's model from simulated data
w = normrnd(0, 1, [1,50]);
[xhat, x, u] = Pred(F, eye(p+1), x0, 50, zeros(1, 50), w);
m = ar(x(2,:), p); 
alpha = [0, -m.A(2:end)];
Ahat = diag(ones(1,p),-1);
Ahat(1,1) = 1;
Ahat(2,:) = alpha;

l = 10;
maxiter = 1000;
tol = 10^(-4);

sim = 50;

balance = 10^(-1);

bar = 1; % xbar is set to be 1 when the prediction is considered by attacker

% set lambda to balance the different sum number
lambda = @(beta) balance*sum(sum(beta))/T;

% when attack target is to change tomorrow's prediction
beta_tom = diag(ones(1,T),1);
xbar_tom = diag(0.05*[1:T].*(T - [1:T]),1);

% initialize cost
cost_mpc_tom = zeros(1, sim);
cost_G_tom = zeros(1, sim);
cost_non_tom = zeros(1, sim);

for iSim = 1:sim
	w = normrnd(0, 0.1, [1, T]);	
	
	% when there is no attack, compute prediction, state value, control sequence (zeros)
	[xhat_non, x_non, u_non] = Pred(F, Ahat, x0, T, zeros(1, T), w);	

	% ----------------------------------------------------------------------------------
	% when attack target is to change tomorrow's prediction	

	% compute policy functions of MPC and Greedy
	[u_mpc_tom, x_mpc_tom] = MPC(F, l, Ahat, xbar_tom, beta_tom, lambda(beta_tom), T, x0, w, maxiter, tol);
	[u_G_tom, x_G_tom] = GreedyNonlinear(F, Ahat, xbar_tom, beta_tom, lambda(beta_tom), T, x0, w);	

	[xhat_mpc_tom, x_mpc_tom, u_mpc_tom] = Pred(F, Ahat, x0, T, u_mpc_tom, w);
	[xhat_G_tom, x_G_tom, u_G_tom] = Pred(F, Ahat, x0, T, u_G_tom, w);

	cost_mpc_tom(iSim) = Cost(Ahat, xbar_tom, beta_tom, x_mpc_tom, u_mpc_tom, lambda(beta_tom), T);
	cost_G_tom(iSim) = Cost(Ahat, xbar_tom, beta_tom, x_G_tom, u_G_tom, lambda(beta_tom), T);
	cost_non_tom(iSim) = Cost(Ahat, xbar_tom, beta_tom, x_non, u_non, lambda(beta_tom), T);	

end


figtoy3 = figure
% when there is no attacker
ax1 = subplot(1,2,1);
for t = 0 : T-1
	p1 = plot([t+1:T], xhat_non(t+1, t+2:T+1), '-b.'); hold on;
end
p2 = plot([0:T], x_non(2,:), '-m.');

xlabel('time');
axis('square');
title(['no attack']);
legend([p1, p2], {'forecast $\hat x_{t'' \mid t} $ ', 'env state $x_t$'}, 'Location', 'northwest','interpreter','latex');

% when attack target is to change tomorrow's prediction
ax2 = subplot(1,2,2);
hold on;
p1 = plot([1:T], diag(xhat_mpc_tom, 1), '-b^');
p2 = plot([0:T], x_mpc_tom(2,:), '-m.');
p3 = plot([0:T-1], u_mpc_tom, '-r.');
p4 = plot([1:T], diag(xbar_tom, 1), '-g.');
p5 = plot([1:T], diag(xhat_G_tom, 1), '--b+');
p6 = plot([0:T], x_G_tom(2,:), ':m.');
p7 = plot([0:T-1], u_G_tom, ':r.');

legend([p1, p2, p3, p4, p5, p6, p7], {'MPC $\hat x_{t+1 \mid t} $', '(M)env $x_t$', 'MPC $u_t$', ...
	'target $\bar x_{t+1 \mid t}$','Greedy $\hat x_{t+1 \mid t}$','(G)env $x_t$', 'Greedy $u_t$'}, ...
	'Location', 'northeast','interpreter','latex','NumColumns',2);
str0 = ['attack $\hat x_{t+1 \mid t}$'];
str1 = ['MPC cost = ', num2str(cost_mpc_tom(sim))];
str2 = ['Greedy cost = ', num2str(cost_G_tom(sim))]; 
str3 = ['no attack cost = ', num2str(cost_non_tom(sim))];
str = {str0;};
title(str,'interpreter','latex');
axis('square');

linkaxes([ax1,ax2],'xy')

saveas(figtoy3,'figexper4_MPCtoy','epsc');

% report mean cost and run paired t-test for 50 pairs of realized cost under MPC and Greedy policy
disp({str0;str1;str2;str3});
disp(' ');
disp(' ');
disp('average cost and t-test when:');
disp(' ');
disp('attack tomorrow''s prediction:');
disp({'no attack cost: = ', mean(cost_non_tom),std(cost_non_tom)/sqrt(length(cost_non_tom))});
disp({'mpc cost: = ', mean(cost_mpc_tom),std(cost_mpc_tom)/sqrt(length(cost_mpc_tom))});
disp({'Greedy cost: = ', mean(cost_G_tom),std(cost_mpc_tom)/sqrt(length(cost_mpc_tom))});
disp('paired t-test, H0: cost_mpc = cost_G; H1: cost_mpc ~= cost_G;');
[h,p] = ttest(cost_mpc_tom,cost_G_tom);
disp({'p-value = ', p});

function varargout = Ftoy(x,u,w,mode)

if mode == 1
	varargout = {[1; 2*x(2)/(1+0.8*x(2)^2) + u + w]};
end
if mode == 2
	Dx = diag(ones(1,1),-1);
	Dx(2,:) = [0,(2 - 1.6 * x(2)^2)/(1 + 0.8 * x(2)^2)^2];
	varargout = {Dx};
end
if mode == 3
	varargout = {[0;1]};
end

end
