% this code is used to generate plot 'figexper6_GNP.eps' for the GNP data in section 4.6.
% this code shows results for paired t-test between realized cost under MPC and Greedy policy
% F is the environment dynamic
% Ahat is forecast matrix
% x0 is the scalar initial state
% T is the time horizon

clear;

rng(1);

p = 2;
T = 10;


F = @(x, u, w, mode) FGNP(x, u, w, mode);

Ahat = diag(ones(1,p),-1);
Ahat(2,:) = [0.0041,0.33,0.13];


x0 = 0.0065;

l = 5;
maxiter = 1000;
tol = 10^(-4);

sim = 50;

balance = 10^(-3);

bar = 0.01; 

% set lambda to balance the different sum number
lambda = @(beta) balance*sum(sum(beta))/T;

beta_final = zeros(T+1, T+1);
beta_final(1:T, T+1) = ones(T, 1);
xbar_final = bar * beta_final;

% initialize cost
cost_mpc_final = zeros(1, sim);
cost_G_final = zeros(1, sim);
cost_non_final = zeros(1, sim);

for iSim = 1:sim
	iSim
	w = normrnd(0, 1, [1, T]);	
	
	% when there is no attack, compute prediction, state value, control sequence (zeros)
	[xhat_non, x_non, u_non] = Pred(F, Ahat, x0, T, zeros(1, T), w);	

	% compute policy functions of MPC and Greedy
	[u_mpc_final, x_mpc_final] = MPC(F, l, Ahat, xbar_final, beta_final, lambda(beta_final), T, x0, w, maxiter, tol);
	[u_G_final, x_G_final] = GreedyNonlinear(F, Ahat, xbar_final, beta_final, lambda(beta_final), T, x0, w);	%
	[xhat_mpc_final, x_mpc_final, u_mpc_final] = Pred(F, Ahat, x0, T, u_mpc_final, w);
	[xhat_G_final, x_G_final, u_G_final] = Pred(F, Ahat, x0, T, u_G_final, w);%
	cost_mpc_final(iSim) = Cost(Ahat, xbar_final, beta_final, x_mpc_final, u_mpc_final, lambda(beta_final), T);
	cost_G_final(iSim) = Cost(Ahat, xbar_final, beta_final, x_G_final, u_G_final, lambda(beta_final), T);
	cost_non_final(iSim) = Cost(Ahat, xbar_final, beta_final, x_non, u_non, lambda(beta_final), T);	%		

end





figreal3 = figure

% when there is no attackers
ax1 = subplot(1,2,1);
for t = 0 : T-1
	p1 = plot([t+1:T], xhat_non(t+1, t+2:T+1), '-b.'); hold on;
end
p2 = plot([0:T], x_non(2,:), '-m.');

xlabel('time');
title(['no attack']);
axis('square');
legend([p1, p2], {'forecast $\hat x_{t'' \mid t} $ ', 'env state $x_t$'}, 'Location', 'northeast','interpreter','latex');% ----------------------------------------------------------------------------------

% plot under MPC and Greedy policy
ax2 = subplot(1,2,2);
[p1, p2, p3] = Predplot(xhat_mpc_final, x_mpc_final, u_mpc_final, T);
p4 = plot([1:T], xbar_final(T, T+1)*ones(1,T), '-g.');
for t = 0 : T-1
	p5 = plot([t+1:T], xhat_G_final(t+1, t+2:T+1), ':b.'); hold on;
end
p6 = plot([0:T], x_G_final(2,:), ':m.');
p7 = plot([0:T-1], u_G_final, ':r.');
legend([p1, p2, p3, p4, p5, p6, p7], {'MPC $\hat x_{t'' \mid t} $', '(M)env $x_t$', 'MPC $u_t$', 'target $\bar x_{T \mid t}$',...
	'Greedy $\hat x_{t'' \mid t}$','(G)env $x_t$', 'Greedy $u_t$'}, 'Location', 'northeast','interpreter','latex');
str0 = ['attack $\hat x_{T \mid t}$'];
str1 = ['MPC cost = ', num2str(cost_mpc_final(sim))];
str2 = ['Greedy cost = ', num2str(cost_G_final(sim))]; 
str3 = ['no attack cost = ', num2str(cost_non_final(sim))];
str = {str0};
title(str,'interpreter','latex');
axis('square');
linkaxes([ax1,ax2],'xy')

saveas(figreal3,'figexper6_GNP','epsc');

% report mean cost and run paired t-test for 50 pairs of realized cost under MPC and Greedy policy
disp({str0;str1;str2;str3});
disp(' ');
disp(' ');
disp('average cost and t-test when:');
disp(' ');
disp('attack final prediction:');
disp({'no attack cost: = ', mean(cost_non_final),std(cost_non_final)/sqrt(length(cost_non_final))});
disp({'mpc cost: = ', mean(cost_mpc_final),std(cost_mpc_final)/sqrt(length(cost_mpc_final))});
disp({'Greedy cost: = ', mean(cost_G_final),std(cost_G_final)/sqrt(length(cost_G_final))});
disp('paired t-test, H0: cost_mpc = cost_G; H1: cost_mpc ~= cost_G;');
[h,p] = ttest(cost_mpc_final,cost_G_final);
disp({'p-value = ', p});


function varargout = FGNP(x,u,w,mode)

if x(2) <= x(3) && x(3) <= 0
	
	if mode == 1
		varargout = { [1; - 0.015 - 1.076 * x(2) + u + 0.0062*w; x(2)]};
	end
	if mode == 2
		Dx = diag(ones(1,2),-1);
		Dx(2,:) = [0,-1.076,0];
		varargout = {Dx};
	end
	if mode == 3
		varargout = {[0;1;0]};
	end

elseif x(2) > x(3) && x(3) <= 0

	if mode == 1
		varargout = { [1; - 0.006 + 0.630 * x(2) - 0.756 * x(3) + u + 0.0132*w; x(2)]};
	end
	if mode == 2
		Dx = diag(ones(1,2),-1);
		Dx(2,:) = [0, 0.630, - 0.756];
		varargout = {Dx};
	end
	if mode == 3
		varargout = {[0;1;0]};
	end

elseif x(2) <= x(3) && x(3) > 0

	if mode == 1
		varargout = { [1; 0.006 + 0.438 * x(2) + u + 0.0094*w; x(2)]};
	end
	if mode == 2
		Dx = diag(ones(1,2),-1);
		Dx(2,:) = [0, 0.438, 0];
		varargout = {Dx};
	end
	if mode == 3
		varargout = {[0;1;0]};
	end

else

	if mode == 1
		varargout = { [1; 0.004 + 0.443 * x(2) + u + 0.0082*w; x(2)]};
	end
	if mode == 2
		Dx = diag(ones(1,2),-1);
		Dx(2,:) = [0, 0.443, 0];
		varargout = {Dx};
	end
	if mode == 3
		varargout = {[0;1;0]};
	end

end

end


function [p1, p2, p3] = Predplot(xhat, x, u, T)
for t = 0 : T-1
	p1 = plot([t+1:T], xhat(t+1, t+2:T+1), '-b.'); hold on;
end
p2 = plot([0:T], x(2,:), '-m.');
p3 = plot([0:T-1], u, '-r.');

xlabel('time');
ylabel('state value');

end